//Code adapted from https://gist.github.com/xmichaelx/39fa023f9ce1fb82d5c0
function createField(width, height) {
	var field = new Uint8Array(width * height);
	var fcenterX = Math.floor(width / 2);
	var fcenterY = Math.floor(height / 2);

	// center occupied
	field[fcenterX + fcenterY * width] = 1;

	return { field: field, width:width, height:height};
}

function createParticles(field, particleCount) {

	var particlesX = new Uint16Array(particleCount);
	var particlesY = new Uint16Array(particleCount);
	var particlesStuck = new Uint8Array(particleCount);

	return { x: particlesX, y:particlesY, stuck: particlesStuck, count: particleCount};
}


function resetSingleParticle(particles, field, i, width, height) {
	var x = Math.floor(Math.random() * width);
	var y = Math.floor(Math.random() * height);

	while(field[y*width+x]) {
		x = Math.floor(Math.random() * width);
		y = Math.floor(Math.random() * height);
	}

	particles.x[i] = x;
	particles.y[i] = y;
}


function resetAllParticles(particles, field, width, height) {
	for (var i=0;i<particles.count;i++) {
		resetSingleParticle(particles, field, i, width, height);
	}
}

function alone(particles, field, i) {
	var cx = particles.x[i];
	var cy = particles.y[i];

	var lx = cx - 1;
	var rx = cx + 1;
	var ty = cy - 1;
	var by = cy + 1;

	if (cx <= 0 || cx >= field.width || 
		lx <= 0 || lx >= field.width || 
		rx <= 0 || rx >= field.width || 
		cy <= 0 || cy >= field.height || 
		ty <= 0 || ty >= field.height || 
		by <= 0 || by >= field.height) {
		return true;
	}
		
	cy *= field.width;     
	by *= field.width;     
	ty *= field.width;

	if (field.field[cx + ty] ||         
		field.field[lx + cy] ||         
		field.field[rx + cy] ||         
		field.field[cx + by]) {
		return false;
	}

	if (field.field[lx + ty] ||         
		field.field[lx + by] ||         
		field.field[rx + ty] ||         
		field.field[rx + by]) {
		return false;
	}

	return true;
}

function updateSingleParticle(particles,field, i) {
	if (particles.stuck[i] == 0) {
		var x = particles.x[i];
		var y = particles.y[i];

		x += Math.random() > 0.5 ? 1 : -1;
		y += Math.random() > 0.5 ? 1 : -1;

		if ( (x < 0) || (y < 0) || (x >= field.width) || (y >= field.height)) {
			resetSingleParticle(particles, field, i);
		}

        particles.x[i] = x;
        particles.y[i] = y;
        
		if (!alone(particles, field, i)) {
			particles.stuck[i] = 1;
			field.field[y*field.width + x] = 1;
		}
	}
}

function step(particles, field) {
	for (var i = 0;i<particles.count;i++) {
		updateSingleParticle(particles, field, i);
	}
}

function createCanvas(width, height, div) {
	var canvas = document.createElement("canvas");

	canvas.width = width;
	canvas.height = height;
	div.appendChild(canvas);
	return canvas;
}

function createWidthSpace(width, height, div) {
	var canvas = document.createElement("canvas");

	div.appendChild(canvas);
	canvas.width = width;
	canvas.height = height;

	return canvas;
}

function createParticleSpace(width, height, div) {
	var canvas = document.createElement("canvas");
	div.appendChild(canvas);
	canvas.width = width;
	canvas.height = height;

	return canvas;
}

function paint(field, canvas, particles) {
	var ctx = canvas.getContext('2d');
	ctx.clearRect ( 0 , 0 , field.width, field.height );
	var imageData = ctx.getImageData(0, 0, field.width, field.height);

	var data = imageData.data;

	for (var i=0;i<particles.count;i++) {
		var x = particles.x[i];
		var y = particles.y[i];
		var stuck = particles.stuck[i];

		var fieldIndex = y * field.width + x;
    	var index = fieldIndex * 4;
    	var value = stuck ? 127 : 0;
    	data[index]   = value;	// red
        data[++index] = 0//value;	// green
        data[++index] = 0//value;	// blue
        data[++index] = 255;	// alpha
	}

	ctx.putImageData(imageData, 0, 0);
}






function startSimulation(div) {
var width = Math.round(document.getElementById("widthPlace").value);
var height = Math.round(document.getElementById("widthPlace").value);
var particleCount = Math.round(document.getElementById("spacePlace").value);

var canvas = createCanvas(width, height, div);
var field = createField(width, height);
var particles = createParticles(field, particleCount);
document.getElementById("partCount").innerHTML = "Number of particles: "+particleCount;
document.getElementById("widthCount").innerHTML = "Width of box/pixel size: "+width;


resetAllParticles(particles,field, width, height);
setInterval(function() {
	for (var i = 0;i < 4;i++) {
		step(particles, field);
	}

	paint(field, canvas, particles);
} , 33);

}

document.getElementById("btnAdd").onclick = function() {
  startSimulation(document.getElementById("simulationPlace"),document.getElementById("widthPlace"),document.getElementById("spacePlace"));
};
